// ----------------------------------------------------------------------------
// DOMSerialize.java
//
// This example source file illustrates how to refit applications that use DOM
// objects for use with the DataPower XA35 API, which does implement DOM support.
//
// The example class DOMSerialize assumes a TransformerFactory member variable
// and defines two methods, directTransform() and workaroundTransform().  If
// you are performing transforms on DOM objects, the directTransform() method
// should look something like what you have today.  The workaroundTransform()
// method shows the steps needed to adapt your code to the XA35 API by going
// through a process of serialization and deserialization.
//
// The serializer used in this example file is Xalan's SerializerToXML class
// (see complete documentation at
// http://xml.apache.org/xalan-j/apidocs/org/apache/xalan/serialize/SerializerToXML.html).
// Any serializer may be used, as long as can convert a DOM object to an
// InputStream or a Reader.
//
// Note that the parser used to de-serialize the XA35's character stream result
// back to a DOM result is also a third-party library.  See
// http://java.sun.com/xml/jaxp/dist/1.1/docs/api/javax/xml/parsers/DocumentBuilderFactory.html
// for complete documentation on how a parser is selected at runtime.
//
// ----------------------------------------------------------------------------


// Import all DataPower XA35 classes
import com.datapower.xml.xslt.xa.*;

// Import Xalan's SerializerToXML class
import org.apache.xalan.serialize.*;

// Other common imports
import javax.xml.transform.*;
import javax.xml.transform.dom.*;
import javax.xml.transform.stream.*;
import javax.xml.parsers.*;
import org.xml.sax.*;
import org.w3c.dom.*;
import java.io.*;

public class DOMSerialize
{
    /**
     * mTransformerFactory is the only member variable assumed to be
     * initialized externally to the two methods defined below.
     **/
    TransformerFactory mTransformerFactory;


    /**
     * directTransform() shows the basic steps needed to do a transform on DOM
     * objects if your TransformerFactory implementation supports them
     * directly.
     **/
    void directTransform (DOMSource domStylesheet, 
                          DOMSource domInput, 
                          DOMResult domOutput)
    throws Exception
    {
        // Construct a transformer directly from the stylesheet source
        Transformer transformer = mTransformerFactory.newTransformer (domStylesheet);

        // Run the transform directly from the input source to the output result
        transformer.transform (domInput, domOutput);
    }


    /**
     * workaroundTransform() takes the same DOM parameters but serializes them
     * to streams which the XA35 can use.  Later, it is shown how the
     * stream-based result from the XA35 is converted back to a DOM result.
     **/
    void workaroundTransform (DOMSource domStylesheet,
                              DOMSource domInput,
                              DOMResult domOutput)
    throws Exception
    {
        SerializerToXML serializer = new SerializerToXML ();
        Writer tmpWriter;
        Reader tmpReader;

        // Construct a transformer indirectly from the stylesheet source:
        //   1) Serialize the stylesheet from a DOMSource to a StringWriter;
        //   2) Convert the StringWriter to a StringReader and from there to a
        //      StreamSource;
        //   3) Construct the Transformer from the StreamSource.
        tmpWriter = new StringWriter ();
        serializer.setWriter (tmpWriter);
        serializer.serialize (domStylesheet.getNode ());
        tmpReader = new StringReader (tmpWriter.toString ());
        Source streamStylesheet = new StreamSource (tmpReader);
        Transformer transformer = mTransformerFactory.newTransformer (streamStylesheet);

        // Follow a similar pattern to convert the input document from a
        // DOMSource to a StreamSource.
        tmpWriter = new StringWriter ();
        serializer.setWriter (tmpWriter);
        serializer.serialize (domInput.getNode ());
        tmpReader = new StringReader (tmpWriter.toString ());
        Source streamInput = new StreamSource (tmpReader);

        // Prepare a StringWriter-based StreamResult as a temporary result.
        tmpWriter = new StringWriter ();
        Result streamOutput = new StreamResult (tmpWriter);

        // Run the transform using the converted input source and the temporary
        // result.
        transformer.transform (streamInput, streamOutput);

        // Convert the result StringWriter to a StringReader and then to a new
        // InputSource, and then parse that InputSource to recover a DOM
        // representation of the result.  
        tmpReader = new StringReader (tmpWriter.toString ());
        InputSource outInputSource = new InputSource (tmpReader);
        DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance ();
        DocumentBuilder db = dbf.newDocumentBuilder ();
        Document result = db.parse (outInputSource);
        domOutput.setNode (result);
    }

}
