// ----------------------------------------------------------------------------
// Example.java
//
// This example source file illustrates how to adjust existing JAXP client code
// to integrate the DataPower JAXP classes for use with the DataPower XA35.
// Each example consists of an oldcode() method that shows what your existing
// code might look like using Xalan as an example, followed by a newcode()
// method that shows 1) selecting the DataPower TransformerFactory
// implementation and 2) configuring the factory with some DataPower-specific
// attributes.
//
// There are three techniques for specifying your desired JAXP implementation.
// The Example1 class shows the changes needed if you use the 'new' operator.
// Example2 shows an example of how to use an implementation-specific
// 'newInstance' method.  Example3 shows how to use the default 'newInstance'
// method in combination with an external environment variable that specifies
// the implementation.  
//
// However the DataPower TransformerFactory is instantiated, there is some
// additional state that must be configured with the setAttribute() method.  In
// each example class, the code for setting these attributes is the same.
//
// ----------------------------------------------------------------------------


// In your current code, you import some other vendor's implementation.  We'll
// use Xalan as an example:
import org.apache.xalan.processor.TransformerFactoryImpl;

// To integrate the DataPower classes, use this import statement:
import com.datapower.xml.xslt.xa.*;

// Other common imports:
import javax.xml.transform.*;
import javax.xml.transform.stream.*;


// ----------------------------------------------------------------------------
// Example1 shows the style of instantiating a particular JAXP implementation
// directly with the 'new' operator.
class Example1
{
    public final void oldcode (String [] args)
    {
        // instantiate the existing TransformerFactory implementation:
        TransformerFactory f = new TransformerFactoryImpl ();
    }

    public final void newcode (String [] args)
    {
        // instantiate the DataPower factory implementation and set the IP
        // address and port of the XA35 to which it should connect:
        TransformerFactory f = new xaTransformerFactory ();
        f.setAttribute ("XAIP", "x.x.x.x");
        f.setAttribute ("XAPort", "nnn");
    }
}

// ----------------------------------------------------------------------------
// Example2 shows the style of calling an implementation-specific 'newInstance'
// method to acquire a new TransformerFactory.
class Example2
{
    public final void oldcode (String [] args)
    {
        // instantiate the existing TransformerFactory implementation:
        TransformerFactory f = TransformerFactoryImpl.newInstance ();
    }

    public final void newcode (String [] args)
    {
        // instantiate the DataPower factory implementation and set the IP
        // address and port of the XA35 to which it should connect:
        TransformerFactory f = xaTransformerFactory.newInstance ();
        f.setAttribute ("XAIP", "x.x.x.x");
        f.setAttribute ("XAPort", "nnn");
    }
}

// ----------------------------------------------------------------------------
// Example3 uses the default TransformerFactory 'newInstance' method and
// specifies the desired implementation externally through an environment
// variable.
class Example3
{
    // Externally, perhaps in a shell script, the user sets an environment
    // variable to specify the complete class name of the existing
    // TransformerFactory implementation:
    //
    // javax.xml.transform.TransformerFactory=org.apache.xalan.processor.TransformerFactoryImpl
    //
    public final void oldcode (String [] args)
    {
        // instantiate the existing TransformerFactory implementation:
        TransformerFactory f = TransformerFactory.newInstance ();
    }

    // To use the DataPower classes, the new value of the environment variable
    // should be the following:
    //
    // javax.xml.transform.TransformerFactory=com.datapower.xml.xa.xaTransformerFactory
    //
    public final void newcode (String [] args)
    {
        // instantiate the DataPower factory implementation and set the IP
        // address and port of the XA35 to which it should connect:
        TransformerFactory f = TransformerFactory.newInstance ();
        f.setAttribute ("XAIP", "x.x.x.x");
        f.setAttribute ("XAPort", "nnn");
    }
}

